#' ECOSolveR: Embedded Conic Solver in R
#'
#' \code{ECOSolveR} is a wrapper around the ecos library.
#' Please see the examples and documentation for the function \code{ECOS_csolve}.
#'
#' @references \url{https://github.com/embotech/ecos}
#' @useDynLib ECOSolveR
#' @docType package
#' @name ECOSolveR
#'
NULL
